"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
const apache_arrow_1 = require("apache-arrow");
class DataPipeModel extends base_1.WidgetModel {
    constructor() {
        super(...arguments);
        this.data = {};
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: DataPipeModel.model_name, _model_module: DataPipeModel.model_module, _model_module_version: DataPipeModel.model_module_version, port: 0 });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.on("change:port", () => {
            this.connectTo(this.get('port'));
        }, this);
    }
    connectTo(port) {
        console.log(`Connecting to ${port}`);
        const socket = new WebSocket(`ws://localhost:${port}`);
        this.socket = socket;
        socket.onopen = () => {
            socket.send('{"body":"My name is John"}');
        };
        new Transfer(socket, this.data);
        socket.onclose = (event) => {
            if (event.wasClean) {
                console.log(`[close] Connection closed cleanly, code=${event.code} reason=${event.reason}`);
            }
        };
        socket.onerror = (error) => {
            console.log(`[error] ${error}`);
        };
    }
}
exports.DataPipeModel = DataPipeModel;
DataPipeModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
DataPipeModel.model_name = 'DataPipeModel';
DataPipeModel.model_module = version_1.MODULE_NAME;
DataPipeModel.model_module_version = version_1.MODULE_VERSION;
class Transfer {
    constructor(socket, data) {
        this._buffer = [];
        this._received = 0;
        this._data = data;
        this._socket = socket;
        socket.onmessage = this.init_msg.bind(this);
    }
    init_msg(event) {
        console.log(event);
        if (event.data instanceof Blob) {
            console.log("[DataPipe} received chunk");
            this._buffer[this._received] = event.data;
            this._received += 1;
            if (this._received == this._current_obj.chunks) {
                console.log(`[DataPipe] transfer of chunks of object ${this._current_obj.id} completed`);
                const table = apache_arrow_1.Table.from(this._buffer);
                this._resolve(table);
                console.log(this._buffer);
                console.log(table);
                this._received = 0;
            }
        }
        else {
            this._current_obj = JSON.parse(event.data);
            const { id, chunks } = this._current_obj;
            this._data[id] = new Promise((resolve, reject) => {
                this._resolve = resolve;
                this._socket.onerror = this._reject;
            });
            this._buffer = new Array(chunks);
            this._received = 0;
            console.log(`[DataPipe} starting transmission ${id} chunk`);
        }
    }
}
//# sourceMappingURL=binary_transfer.js.map