"use strict";
// Copyright (c) Cesare Cugnasco
// Distributed under the terms of the Modified BSD License.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const chroma_js_1 = __importDefault(require("chroma-js"));
const version_1 = require("./version");
// Import the CSS
require("../css/widget.css");
const THREE = __importStar(require("three"));
const OrbitControls_1 = require("three/examples/jsm/controls/OrbitControls");
const PARTICLE_SIZE = 20;
class ExampleModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ExampleModel.model_name, _model_module: ExampleModel.model_module, _model_module_version: ExampleModel.model_module_version, _view_name: ExampleModel.view_name, _view_module: ExampleModel.view_module, _view_module_version: ExampleModel.view_module_version, value: 'Hello World' });
    }
}
exports.ExampleModel = ExampleModel;
ExampleModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
ExampleModel.model_name = 'ExampleModel';
ExampleModel.model_module = version_1.MODULE_NAME;
ExampleModel.model_module_version = version_1.MODULE_VERSION;
ExampleModel.view_name = 'ExampleView'; // Set to null if no view
ExampleModel.view_module = version_1.MODULE_NAME; // Set to null if no view
ExampleModel.view_module_version = version_1.MODULE_VERSION;
class ExampleView extends base_1.DOMWidgetView {
    render() {
        this.el.classList.add('custom-widget');
        this.value_changed();
        // Python -> JavaScript update
        this.model.on('change:value', this.value_changed, this);
    }
    value_changed() {
        this.el.textContent = this.model.get('value');
    }
}
exports.ExampleView = ExampleView;
class SourceModel extends base_1.WidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SourceModel.model_name, _model_module: SourceModel.model_module, _model_module_version: SourceModel.model_module_version, keyspace: null, table_name: null, indexed_columns: [], columns: [] });
    }
}
exports.SourceModel = SourceModel;
SourceModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
SourceModel.model_name = 'SourceModel';
SourceModel.model_module = version_1.MODULE_NAME;
SourceModel.model_module_version = version_1.MODULE_VERSION;
class QvizModel extends base_1.WidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: QvizModel.model_name, _model_module: QvizModel.model_module, _model_module_version: QvizModel.model_module_version, _view_name: ExampleModel.view_name, _view_module: ExampleModel.view_module, _view_module_version: ExampleModel.view_module_version });
    }
}
exports.QvizModel = QvizModel;
QvizModel.serializers = Object.assign(Object.assign({}, base_1.DOMWidgetModel.serializers), { source: { deserialize: base_1.unpack_models } });
QvizModel.model_name = 'QVizModel';
QvizModel.model_module = version_1.MODULE_NAME;
QvizModel.model_module_version = version_1.MODULE_VERSION;
QvizModel.view_name = 'QvizView';
QvizModel.view_module = version_1.MODULE_NAME;
QvizModel.view_module_version = version_1.MODULE_VERSION;
class QvizView extends base_1.DOMWidgetView {
    initialize(parameters) {
        console.log("Qviz initialized");
    }
    render() {
        console.log('loading TableView reader');
        this.qviz = new Qviz(this.el);
        /*this.el.classList.add('custom-widget');
        this.red_div = document.createElement('p');
        this.red_div.style.backgroundColor = 'red';

        this.el.appendChild(this.red_div);

        this.value_changed();
        // Python -> JavaScript update
        this.model.on('change:source', this.value_changed, this);
        this.model.get('source').on('change:keyspace', this.value_changed, this);*/
        this.model.on('change:data', this.data_changed, this);
    }
    data_changed() {
        const source = this.model.get("data");
        const colorScale = chroma_js_1.default.scale(['green', 'red', 'navy']).mode('hsl');
        const x = Object.values(source.x);
        const y = Object.values(source.y);
        const z = Object.values(source.z);
        // @ts-ignore
        const colors = source.t ? Object.values(source.t).map(colorScale).map(a => a._rgb) :
            Array.from({ length: z.length }, () => [Math.random() * 100, Math.random() * 100, Math.random() * 100]);
        const data = {
            positions: x.map((a, i) => [a, y[i], z[i]]),
            colors: colors,
        };
        this.qviz.addData(data);
    }
    value_changed() {
        this.red_div.textContent = "super! " + this.model.get('source').get('keyspace');
    }
}
exports.QvizView = QvizView;
class Qviz {
    constructor(destination) {
        this.scene = new THREE.Scene();
        this.camera = new THREE.PerspectiveCamera(45, destination.clientWidth / destination.clientHeight, 1, 10000);
        this.camera.position.z = 250;
        //
        //
        this.renderer = new THREE.WebGLRenderer();
        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(window.innerWidth, window.innerHeight);
        //
        this.raycaster = new THREE.Raycaster();
        this.mouse = new THREE.Vector2();
        //
        //stats = new Stats();
        //container.appendChild(stats.dom);
        //
        console.log("Qviz initialized");
        destination.appendChild(this.renderer.domElement);
        const controls = new OrbitControls_1.OrbitControls(this.camera, this.renderer.domElement);
        controls.enableDamping = true; // an animation loop is required when either damping or auto-rotation are enabled
        controls.dampingFactor = 0.05;
        controls.screenSpacePanning = false;
        controls.minDistance = 100;
        controls.maxDistance = 500;
        controls.maxPolarAngle = Math.PI / 2;
        this.controls = controls;
        // Here all listerners
        this.onWindowResize();
        window.addEventListener('resize', this.onWindowResize.bind(this), false);
        //destination.addEventListener('resize', this.onWindowResize.bind(this), false);
        //document.addEventListener('mousemove', this.onDocumentMouseMove, false);
        this.animate();
    }
    addData(points, reset = false) {
        const positions = new Float32Array(points.positions.length * 3);
        const colors = new Float32Array(points.positions.length * 3);
        const sizes = new Float32Array(points.positions.length);
        const geometry = new THREE.BufferGeometry();
        const material = new THREE.PointsMaterial({ vertexColors: true });
        if (reset) {
            this.scene.remove(this.particles);
        }
        this.particles = new THREE.Points(geometry, material);
        const vertex = new THREE.Vector3();
        const color = new THREE.Color();
        for (var i = 0, l = points.positions.length; i < l; i++) {
            const [x, y, z] = points.positions[i];
            vertex.set(x, y, z);
            vertex.toArray(positions, i * 3);
            const [r, g, b] = points.colors[i];
            color.setRGB(r, g, b);
            color.toArray(colors, i * 3);
            sizes[i] = PARTICLE_SIZE * 0.5;
        }
        geometry.setAttribute('position', new THREE.BufferAttribute(positions, 3));
        geometry.setAttribute('color', new THREE.BufferAttribute(colors, 3));
        geometry.setAttribute('size', new THREE.BufferAttribute(sizes, 1));
        this.scene.add(this.particles);
    }
    /*    onDocumentMouseMove(event) {
            console.log("onDocumentMouseMove")

            event.preventDefault();

            this.mouse.x = (event.clientX / window.innerWidth) * 2 - 1;
            this.mouse.y = -(event.clientY / window.innerHeight) * 2 + 1;

        }

        resizeRendererToDisplaySize() {
            const canvas = this.renderer.domElement

            const pixelRatio = window.devicePixelRatio;
            const width = canvas.clientWidth * pixelRatio | 0;
            const height = canvas.clientHeight * pixelRatio | 0;
            const needResize = canvas.width !== width || canvas.height !== height;
            if (needResize) {
                console.log("needs to resize!")
                this.renderer.setSize(width, height, false);
                this.camera.aspect = canvas.clientWidth / canvas.clientHeight;
                this.camera.updateProjectionMatrix();
            }

        }*/
    onWindowResize() {
        console.log("resizing!");
        this.camera.aspect = window.innerWidth / window.innerHeight;
        this.camera.updateProjectionMatrix();
        this.renderer.setSize(window.innerWidth, window.innerHeight);
    }
    animate() {
        requestAnimationFrame(this.animate.bind(this));
        this.render();
        //stats.update();
        this.controls.update();
        //  this.resizeRendererToDisplaySize()
    }
    render() {
        //const {raycaster, mouse, camera, renderer, INTERSECTED, scene} = this;
        const { camera, renderer, scene } = this;
        if (this.particles) {
            this.particles.rotation.x += 0.0005;
            this.particles.rotation.y += 0.001;
        }
        /*
                const geometry = particles.geometry;
                // @ts-ignore
                const attributes = geometry.attributes;

                raycaster.setFromCamera(mouse, camera);

                const intersects = this.raycaster.intersectObject(particles);

                if (intersects.length > 0) {

                    if (INTERSECTED != intersects[0].index) {

                        attributes.size.array[INTERSECTED] = PARTICLE_SIZE;

                        this.INTERSECTED = intersects[0].index;

                        attributes.size.array[INTERSECTED] = PARTICLE_SIZE * 1.25;
                        attributes.size.needsUpdate = true;

                    }

                } else if (INTERSECTED !== null) {

                    attributes.size.array[INTERSECTED] = PARTICLE_SIZE;
                    attributes.size.needsUpdate = true;
                    this.INTERSECTED = null;

                }*/
        renderer.render(scene, camera);
    }
}
//# sourceMappingURL=widget.js.map